// ********** TABLES **********

CREATE TABLE `category` (
    _id INTEGER PRIMARY KEY,
    is_visible INTEGER NOT NULL DEFAULT 1,
    name TEXT,
    default_icon_id INTEGER DEFAULT 1,
    default_color_id INTEGER DEFAULT 1,
    default_ringtone TEXT,
    default_vibrate INTEGER NOT NULL DEFAULT 1);
CREATE TABLE `base_data` (
	`_id`	INTEGER,
	`description`	TEXT,
	`icon_id`	INTEGER NOT NULL DEFAULT 1,
	`color_id`	INTEGER NOT NULL DEFAULT 1,
	`category_id`	INTEGER NOT NULL DEFAULT 1,
	PRIMARY KEY(_id),
	FOREIGN KEY(`category_id`) REFERENCES category(_id)
);
CREATE TABLE `note` (
	`_id`	INTEGER,
	`base_data_id`	INTEGER NOT NULL,
	PRIMARY KEY(_id),
	FOREIGN KEY(`base_data_id`) REFERENCES base_data(_id)
);
CREATE TABLE `task` (
	`_id`	INTEGER,
	`base_data_id`	INTEGER NOT NULL,
	`is_done`	INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY(_id),
	FOREIGN KEY(`base_data_id`) REFERENCES base_data(_id)
);
CREATE TABLE `alarm` (
	`_id`	INTEGER,
	`base_data_id`	INTEGER NOT NULL,
	`is_enabled`	INTEGER NOT NULL DEFAULT 1,
	`date_start`	INTEGER NOT NULL,
	`date_end`	INTEGER,
	`occurrences_end`	INTEGER NOT NULL DEFAULT 0,
	`occurrence_counter`	INTEGER NOT NULL DEFAULT 0,
	`ringtone`	TEXT,
	`is_vibrate`	INTEGER NOT NULL DEFAULT 1,
	PRIMARY KEY(_id),
	FOREIGN KEY(`base_data_id`) REFERENCES base_data(_id)
);
CREATE TABLE "alarm_on_time" (
	`_id`	INTEGER,
	`alarm_id`	INTEGER NOT NULL,
	`hour`	INTEGER NOT NULL,
	`minute`	INTEGER NOT NULL,
	`repeat_type`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_monday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_tuesday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_wednesday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_thursday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_friday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_saturday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_sunday`	INTEGER NOT NULL DEFAULT 1,
	`rep_every_number_of_units`	INTEGER NOT NULL DEFAULT 1,
	`rep_unit`	INTEGER NOT NULL DEFAULT 1,
	`next_occurrence`	INTEGER NOT NULL DEFAULT 0,
	`snooze_time`	INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY(_id),
	FOREIGN KEY(`alarm_id`) REFERENCES alarm(_id)
);
CREATE TABLE "alarm_in_place" (
	`_id`	INTEGER,
	`alarm_id`	INTEGER NOT NULL,
	`location_name`	TEXT,
	`longitude`	REAL NOT NULL,
	`latitude`	REAL NOT NULL,
	`radius`	INTEGER NOT NULL DEFAULT 328,
	`snooze_time`	INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY(_id),
	FOREIGN KEY(`alarm_id`) REFERENCES alarm(_id)
);

// ********** VIEWS **********

CREATE VIEW category_view
AS
	SELECT c.*, coalesce(b.cat_cnt,0) as category_usage_count
		FROM category AS c
		LEFT JOIN (
			SELECT category_id, COUNT(*) AS cat_cnt
				FROM base_data
				GROUP BY category_id) AS b
		ON
			c._id=b.category_id;
CREATE VIEW note_view
AS
	SELECT t.*, b.*, c.name as category_name FROM (note t JOIN base_data b
        ON t.base_data_id = b._id)
            JOIN category c
                ON c._id=b.category_id AND c.is_visible=1;
CREATE VIEW task_view
AS
    SELECT t.*, b.*, c.name as category_name FROM (task t JOIN base_data b
        ON t.base_data_id = b._id)
            JOIN category c
                ON c._id=b.category_id AND c.is_visible=1;
CREATE VIEW aot_view
AS
	SELECT t.*, a.*, b.*, c.name as category_name FROM ((alarm_on_time t JOIN alarm a
    		ON t.alarm_id = a._id)
    			JOIN base_data b
    				ON a.base_data_id = b._id)
    					JOIN category c
    						ON c._id=b.category_id AND c.is_visible=1;
CREATE VIEW aip_view
AS
	SELECT t.*, a.*, b.*, c.name as category_name FROM ((alarm_in_place t JOIN alarm a
		ON t.alarm_id = a._id)
			JOIN base_data b
				ON a.base_data_id = b._id)
                    JOIN category c
                        ON c._id=b.category_id AND c.is_visible=1;

// ********** CATEGORY VIEW TRIGGERS **********

CREATE TRIGGER category_is_visible_update
INSTEAD OF UPDATE OF is_visible
ON category_view
BEGIN
	UPDATE category SET is_visible=NEW.is_visible
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_name_update
INSTEAD OF UPDATE OF name
ON category_view
BEGIN
	UPDATE category SET name=NEW.name
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_default_icon_id_update
INSTEAD OF UPDATE OF default_icon_id
ON category_view
BEGIN
	UPDATE category SET default_icon_id=NEW.default_icon_id
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_default_color_id_update
INSTEAD OF UPDATE OF default_color_id
ON category_view
BEGIN
	UPDATE category SET default_color_id=NEW.default_color_id
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_default_ringtone_update
INSTEAD OF UPDATE OF default_ringtone
ON category_view
BEGIN
	UPDATE category SET default_ringtone=NEW.default_ringtone
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_default_vibrate_update
INSTEAD OF UPDATE OF default_vibrate
ON category_view
BEGIN
	UPDATE category SET default_vibrate=NEW.default_vibrate
		WHERE _id=NEW._id;
END;
CREATE TRIGGER category_insert
INSTEAD OF INSERT
ON category_view
BEGIN
	INSERT INTO category (name, default_icon_id, default_color_id, default_ringtone, default_vibrate)
		SELECT NEW.name, NEW.default_icon_id, NEW.default_color_id, NEW.default_ringtone, NEW.default_vibrate;
END;
CREATE TRIGGER category_delete
INSTEAD OF DELETE
ON category_view
BEGIN
	DELETE FROM category WHERE _id=OLD._id;
END;

// ********** NOTES VIEW TRIGGERS **********

CREATE TRIGGER note_description_update
INSTEAD OF UPDATE OF description
ON note_view
BEGIN
	UPDATE base_data SET description=NEW.description
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER note_icon_id_update
INSTEAD OF UPDATE OF icon_id
ON note_view
BEGIN
	UPDATE base_data SET icon_id=NEW.icon_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER note_color_id_update
INSTEAD OF UPDATE OF color_id
ON note_view
BEGIN
	UPDATE base_data SET color_id=NEW.color_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER note_category_id_update
INSTEAD OF UPDATE OF category_id
ON note_view
BEGIN
	UPDATE base_data SET category_id=NEW.category_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER note_insert
INSTEAD OF INSERT
ON note_view
BEGIN
	INSERT INTO base_data (description, icon_id, color_id, category_id)
		SELECT NEW.description, NEW.icon_id, NEW.color_id, NEW.category_id;
	INSERT INTO note (base_data_id)
		SELECT base_data._id
		FROM base_data
		ORDER BY base_data._id DESC
		LIMIT 1;
	UPDATE category SET is_visible = 1 WHERE _id=NEW.category_id;
END;
CREATE TRIGGER note_delete
INSTEAD OF DELETE
ON note_view
BEGIN
	DELETE FROM note WHERE _id=OLD._id;
	DELETE FROM base_data WHERE _id=OLD.base_data_id;
END;

// ********** TASKS VIEW TRIGGERS **********

CREATE TRIGGER task_is_done_update
INSTEAD OF UPDATE OF is_done
ON task_view
BEGIN
	UPDATE task SET is_done=NEW.is_done
		WHERE _id=NEW._id;
END;
CREATE TRIGGER task_description_update
INSTEAD OF UPDATE OF description
ON task_view
BEGIN
	UPDATE base_data SET description=NEW.description
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER task_icon_id_update
INSTEAD OF UPDATE OF icon_id
ON task_view
BEGIN
	UPDATE base_data SET icon_id=NEW.icon_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER task_color_id_update
INSTEAD OF UPDATE OF color_id
ON task_view
BEGIN
	UPDATE base_data SET color_id=NEW.color_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER task_category_id_update
INSTEAD OF UPDATE OF category_id
ON task_view
BEGIN
	UPDATE base_data SET category_id=NEW.category_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER task_insert
INSTEAD OF INSERT
ON task_view
BEGIN
	INSERT INTO base_data (description, icon_id, color_id, category_id)
		SELECT NEW.description, NEW.icon_id, NEW.color_id, NEW.category_id;
	INSERT INTO task (base_data_id, is_done)
		SELECT base_data._id, 0
		FROM base_data
		ORDER BY base_data._id DESC
		LIMIT 1;
	UPDATE category SET is_visible = 1 WHERE _id=NEW.category_id;
END;
CREATE TRIGGER task_delete
INSTEAD OF DELETE
ON task_view
BEGIN
	DELETE FROM task WHERE _id=OLD._id;
	DELETE FROM base_data WHERE _id=OLD.base_data_id;
END;

// ********** AOT VIEW TRIGGERS **********

CREATE TRIGGER aot_hour_update
INSTEAD OF UPDATE OF hour
ON aot_view
BEGIN
	UPDATE alarm_on_time SET hour=NEW.hour
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_minute_update
INSTEAD OF UPDATE OF minute
ON aot_view
BEGIN
	UPDATE alarm_on_time SET minute=NEW.minute
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_occurrences_end_update
INSTEAD OF UPDATE OF occurrences_end
ON aot_view
BEGIN
	UPDATE alarm SET occurrences_end=NEW.occurrences_end
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_occurrence_counter_update
INSTEAD OF UPDATE OF occurrence_counter
ON aot_view
BEGIN
	UPDATE alarm SET occurrence_counter=NEW.occurrence_counter
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_repeat_type_update
INSTEAD OF UPDATE OF repeat_type
ON aot_view
BEGIN
	UPDATE alarm_on_time SET repeat_type=NEW.repeat_type
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_monday_update
INSTEAD OF UPDATE OF rep_every_monday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_monday=NEW.rep_every_monday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_tuesday_update
INSTEAD OF UPDATE OF rep_every_tuesday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_tuesday=NEW.rep_every_tuesday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_wednesday_update
INSTEAD OF UPDATE OF rep_every_wednesday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_wednesday=NEW.rep_every_wednesday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_thursday_update
INSTEAD OF UPDATE OF rep_every_thursday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_thursday=NEW.rep_every_thursday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_friday_update
INSTEAD OF UPDATE OF rep_every_friday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_friday=NEW.rep_every_friday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_saturday_update
INSTEAD OF UPDATE OF rep_every_saturday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_saturday=NEW.rep_every_saturday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_sunday_update
INSTEAD OF UPDATE OF rep_every_sunday
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_sunday=NEW.rep_every_sunday
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_every_number_of_units_update
INSTEAD OF UPDATE OF rep_every_number_of_units
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_every_number_of_units=NEW.rep_every_number_of_units
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_rep_unit_update
INSTEAD OF UPDATE OF rep_unit
ON aot_view
BEGIN
	UPDATE alarm_on_time SET rep_unit=NEW.rep_unit
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_next_occurrence_update
INSTEAD OF UPDATE OF next_occurrence
ON aot_view
BEGIN
	UPDATE alarm_on_time SET next_occurrence=NEW.next_occurrence
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_snooze_time_update
INSTEAD OF UPDATE OF snooze_time
ON aot_view
BEGIN
	UPDATE alarm_on_time SET snooze_time=NEW.snooze_time
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aot_is_enabled_update
INSTEAD OF UPDATE OF is_enabled
ON aot_view
BEGIN
	UPDATE alarm SET is_enabled=NEW.is_enabled
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_date_start_update
INSTEAD OF UPDATE OF date_start
ON aot_view
BEGIN
	UPDATE alarm SET date_start=NEW.date_start
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_date_end_update
INSTEAD OF UPDATE OF date_end
ON aot_view
BEGIN
	UPDATE alarm SET date_end=NEW.date_end
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_ringtone_update
INSTEAD OF UPDATE OF ringtone
ON aot_view
BEGIN
	UPDATE alarm SET ringtone=NEW.ringtone
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_is_vibrate_update
INSTEAD OF UPDATE OF is_vibrate
ON aot_view
BEGIN
	UPDATE alarm SET is_vibrate=NEW.is_vibrate
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aot_description_update
INSTEAD OF UPDATE OF description
ON aot_view
BEGIN
	UPDATE base_data SET description=NEW.description
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aot_icon_id_update
INSTEAD OF UPDATE OF icon_id
ON aot_view
BEGIN
	UPDATE base_data SET icon_id=NEW.icon_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aot_color_id_update
INSTEAD OF UPDATE OF color_id
ON aot_view
BEGIN
	UPDATE base_data SET color_id=NEW.color_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aot_category_id_update
INSTEAD OF UPDATE OF category_id
ON aot_view
BEGIN
	UPDATE base_data SET category_id=NEW.category_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aot_insert
INSTEAD OF INSERT
ON aot_view
BEGIN
	INSERT INTO base_data (description, icon_id, color_id, category_id)
		SELECT NEW.description, NEW.icon_id, new.color_id, NEW.category_id;
	INSERT INTO alarm (base_data_id, is_enabled, date_start, date_end, occurrences_end, occurrence_counter, ringtone, is_vibrate)
		SELECT base_data._id, NEW.is_enabled, NEW.date_start, null, 0, 0, NEW.ringtone, NEW.is_vibrate
		FROM base_data
		ORDER BY base_data._id DESC
		LIMIT 1;
	INSERT INTO alarm_on_time (alarm_id, hour, minute,
			repeat_type, rep_every_monday, rep_every_tuesday, rep_every_wednesday, rep_every_thursday, rep_every_friday, rep_every_saturday, rep_every_sunday,
			rep_every_number_of_units, rep_unit, next_occurrence, snooze_time)
		SELECT alarm._id, NEW.hour, NEW.minute,
			1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0
		FROM alarm
		ORDER BY alarm._id DESC
		LIMIT 1;
	UPDATE category SET is_visible = 1 WHERE _id=NEW.category_id;
END;
CREATE TRIGGER aot_delete
INSTEAD OF DELETE
ON aot_view
BEGIN
	DELETE FROM alarm_on_time WHERE _id=OLD._id;
	DELETE FROM alarm WHERE _id=OLD.alarm_id;
	DELETE FROM base_data WHERE _id=OLD.base_data_id;
END;

// ********** AIP VIEW TRIGGERS **********

CREATE TRIGGER aip_location_name_update
INSTEAD OF UPDATE OF location_name
ON aip_view
BEGIN
	UPDATE alarm_in_place SET location_name=NEW.location_name
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aip_longitude_update
INSTEAD OF UPDATE OF longitude
ON aip_view
BEGIN
	UPDATE alarm_in_place SET longitude=NEW.longitude
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aip_latitude_update
INSTEAD OF UPDATE OF latitude
ON aip_view
BEGIN
	UPDATE alarm_in_place SET latitude=NEW.latitude
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aip_radius_update
INSTEAD OF UPDATE OF radius
ON aip_view
BEGIN
	UPDATE alarm_in_place SET radius=NEW.radius
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aip_snooze_time_update
INSTEAD OF UPDATE OF snooze_time
ON aip_view
BEGIN
	UPDATE alarm_in_place SET snooze_time=NEW.snooze_time
		WHERE _id=NEW._id;
END;
CREATE TRIGGER aip_is_enabled_update
INSTEAD OF UPDATE OF is_enabled
ON aip_view
BEGIN
	UPDATE alarm SET is_enabled=NEW.is_enabled
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_date_start_update
INSTEAD OF UPDATE OF date_start
ON aip_view
BEGIN
	UPDATE alarm SET date_start=NEW.date_start
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_date_end_update
INSTEAD OF UPDATE OF date_end
ON aip_view
BEGIN
	UPDATE alarm SET date_end=NEW.date_end
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_occurrences_end_update
INSTEAD OF UPDATE OF occurrences_end
ON aip_view
BEGIN
	UPDATE alarm SET occurrences_end=NEW.occurrences_end
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_occurrence_counter_update
INSTEAD OF UPDATE OF occurrence_counter
ON aip_view
BEGIN
	UPDATE alarm SET occurrence_counter=NEW.occurrence_counter
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_ringtone_update
INSTEAD OF UPDATE OF ringtone
ON aip_view
BEGIN
	UPDATE alarm SET ringtone=NEW.ringtone
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_is_vibrate_update
INSTEAD OF UPDATE OF is_vibrate
ON aip_view
BEGIN
	UPDATE alarm SET is_vibrate=NEW.is_vibrate
		WHERE _id=NEW.alarm_id;
END;
CREATE TRIGGER aip_description_update
INSTEAD OF UPDATE OF description
ON aip_view
BEGIN
	UPDATE base_data SET description=NEW.description
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aip_icon_id_update
INSTEAD OF UPDATE OF icon_id
ON aip_view
BEGIN
	UPDATE base_data SET icon_id=NEW.icon_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aip_color_id_update
INSTEAD OF UPDATE OF color_id
ON aip_view
BEGIN
	UPDATE base_data SET color_id=NEW.color_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aip_category_id_update
INSTEAD OF UPDATE OF category_id
ON aip_view
BEGIN
	UPDATE base_data SET category_id=NEW.category_id
		WHERE _id=NEW.base_data_id;
END;
CREATE TRIGGER aip_insert
INSTEAD OF INSERT
ON aip_view
BEGIN
	INSERT INTO base_data (description, icon_id, color_id, category_id)
		SELECT NEW.description, NEW.icon_id, NEW.color_id, NEW.category_id;
	INSERT INTO alarm (base_data_id, is_enabled, date_start, date_end, occurrences_end, occurrence_counter, ringtone, is_vibrate)
		SELECT base_data._id, NEW.is_enabled, NEW.date_start, null, 0, 0, NEW.ringtone, NEW.is_vibrate
		FROM base_data
		ORDER BY base_data._id DESC
		LIMIT 1;
	INSERT INTO alarm_in_place (alarm_id, location_name, longitude, latitude, radius, snooze_time)
		SELECT alarm._id, NEW.location_name, NEW.longitude, NEW.latitude,
			NEW.radius, 0
		FROM alarm
		ORDER BY alarm._id DESC
		LIMIT 1;
	UPDATE category SET is_visible = 1 WHERE _id=NEW.category_id;
END;
CREATE TRIGGER aip_delete
INSTEAD OF DELETE
ON aip_view
BEGIN
	DELETE FROM alarm_in_place WHERE _id=OLD._id;
	DELETE FROM alarm WHERE _id=OLD.alarm_id;
	DELETE FROM base_data WHERE _id=OLD.base_data_id;
END;